@@ -1,11 +1,9 @@
 # encoding=utf-8
 '''HTTP communication recorders.'''
 import abc
-import base64
 import contextlib
 import datetime
 import gettext
-import hashlib
 import http.client
 import io
 import itertools
@@ -16,14 +14,15 @@ import sys
 from tempfile import NamedTemporaryFile
 import tempfile
 import time
-import uuid
 
 import wpull.backport.gzip
 from wpull.namevalue import NameValueRecord
 from wpull.network import BandwidthMeter
 import wpull.util
+from wpull.warc import WARCRecord
 
 
+_logger = logging.getLogger(__name__)
 _ = gettext.gettext
 
 
@@ -123,127 +122,32 @@ class DemuxRecorderSession(BaseRecorderSession):
             context.__exit__(*args)
 
 
-class WARCRecord(object):
-    '''A record in a WARC file.'''
-    VERSION = 'WARC/1.0'
-    WARC_TYPE = 'WARC-Type'
-    CONTENT_TYPE = 'Content-Type'
-    WARC_DATE = 'WARC-Date'
-    WARC_RECORD_ID = 'WARC-Record-ID'
-    WARCINFO = 'warcinfo'
-    WARC_FIELDS = 'application/warc-fields'
-    REQUEST = 'request'
-    RESPONSE = 'response'
-    TYPE_REQUEST = 'application/http;msgtype=request'
-    TYPE_RESPONSE = 'application/http;msgtype=response'
-    NAME_OVERRIDES = frozenset([
-        'WARC-Date',
-        'WARC-Type',
-        'WARC-Record-ID',
-        'WARC-Concurrent-To',
-        'WARC-Refers-To',
-        'Content-Length',
-        'Content-Type',
-        'WARC-Target-URI',
-        'WARC-Block-Digest',
-        'WARC-IP-Address',
-        'WARC-Filename',
-        'WARC-Warcinfo-ID',
-        'WARC-Payload-Digest',
-    ])
-    '''Field name case normalization overrides because hanzo's warc-tools do
-    not adequately conform to specifications.'''
-
-    def __init__(self):
-        self.fields = NameValueRecord(normalize_overrides=self.NAME_OVERRIDES)
-        self.block_file = None
-
-    def set_common_fields(self, warc_type, content_type):
-        '''Set the required fields for the record.'''
-        self.fields[self.WARC_TYPE] = warc_type
-        self.fields[self.CONTENT_TYPE] = content_type
-        self.fields[self.WARC_DATE] = wpull.util.datetime_str()
-        self.fields[self.WARC_RECORD_ID] = '<{0}>'.format(uuid.uuid4().urn)
-
-    def compute_checksum(self, payload_offset=None):
-        '''Compute and add the checksum data to the record fields.'''
-        if not self.block_file:
-            self.fields['Content-Length'] = '0'
-            return
-
-        block_hasher = hashlib.sha1()
-        payload_hasher = hashlib.sha1()
-
-        with wpull.util.reset_file_offset(self.block_file):
-            if payload_offset is not None:
-                data = self.block_file.read(payload_offset)
-                block_hasher.update(data)
-
-            while True:
-                data = self.block_file.read(4096)
-                if data == b'':
-                    break
-                block_hasher.update(data)
-                payload_hasher.update(data)
-
-            content_length = self.block_file.tell()
-
-        content_hash = block_hasher.digest()
-
-        self.fields['WARC-Block-Digest'] = 'sha1:{0}'.format(
-            base64.b32encode(content_hash).decode()
-        )
-
-        if payload_offset is not None:
-            payload_hash = payload_hasher.digest()
-            self.fields['WARC-Payload-Digest'] = 'sha1:{0}'.format(
-                base64.b32encode(payload_hash).decode()
-            )
-
-        self.fields['Content-Length'] = str(content_length)
-
-    def __iter__(self):
-        yield self.VERSION.encode()
-        yield b'\r\n'
-        yield bytes(self.fields)
-        yield b'\r\n'
-
-        with wpull.util.reset_file_offset(self.block_file):
-            while True:
-                data = self.block_file.read(4096)
-                if data == b'':
-                    break
-                yield data
-
-        yield b'\r\n\r\n'
-
-    def __str__(self):
-        return ''.join(iter(self))
-
-
 class WARCRecorder(BaseRecorder):
     '''Record to WARC file.
 
-    For the WARC file specification, see
-    http://bibnum.bnf.fr/WARC/WARC_ISO_28500_version1_latestdraft.pdf.
-
     Args:
-        filename: The filename (excluding the extension)
-        compress: If True, files will be compressed with gzip
-        extra_fields: A list of key-value pairs containing extra metadata
-            fields
-        temp_dir: Directory to use for temporary files
-        log: Include the program logging messages in the WARC file
-        appending: If True, the file is not overwritten upon opening
+        filename (str): The filename (including the extension).
+        compress (bool): If True, files will be compressed with gzip
+        extra_fields (list): A list of key-value pairs containing extra
+            metadata fields
+        temp_dir (str): Directory to use for temporary files
+        log (bool): Include the program logging messages in the WARC file
+        appending (bool): If True, the file is not overwritten upon opening
+        digests (bool): If True, the SHA1 hash digests will be written.
+        cdx_filename (str): If given, a CDX file will be written.
     '''
+    CDX_DELIMINATOR = ' '
+
     def __init__(self, filename, compress=True, extra_fields=None,
-    temp_dir=None, log=True, appending=False):
+    temp_dir=None, log=True, appending=False, digests=True, cdx_filename=None):
         self._filename = filename
         self._gzip_enabled = compress
         self._temp_dir = temp_dir
         self._warcinfo_record = WARCRecord()
         self._log_record = None
         self._log_handler = None
+        self._digests_enabled = digests
+        self._cdx_filename = cdx_filename
 
         if not appending:
             self._truncate_existing_file()
@@ -254,13 +158,21 @@ class WARCRecorder(BaseRecorder):
             self._log_record = WARCRecord()
             self._setup_log()
 
+        if self._cdx_filename:
+            self._write_cdx_header()
+
         self.write_record(self._warcinfo_record)
 
     def _truncate_existing_file(self):
+        '''Truncate existing WARC and CDX file if it exists.'''
         if os.path.exists(self._filename):
             with open(self._filename, 'wb'):
                 pass
 
+        if os.path.exists(self._cdx_filename):
+            with open(self._cdx_filename, 'wb'):
+                pass
+
     def _populate_warcinfo(self, extra_fields=None):
         '''Add the metadata to the Warcinfo record.'''
         self._warcinfo_record.set_common_fields(
@@ -306,6 +218,13 @@ class WARCRecorder(BaseRecorder):
         recorder_session = WARCRecorderSession(self, self._temp_dir)
         yield recorder_session
 
+    def set_length_and_maybe_checksums(self, record, payload_offset=None):
+        '''Set the content length and possibly the checksums.'''
+        if self._digests_enabled:
+            record.compute_checksum(payload_offset)
+        else:
+            record.set_content_length()
+
     def write_record(self, record):
         '''Append the record to the WARC file.'''
         # FIXME: probably not a good idea to modifiy arguments passed to us
@@ -313,15 +232,33 @@ class WARCRecorder(BaseRecorder):
         record.fields['WARC-Warcinfo-ID'] = self._warcinfo_record.fields[
             WARCRecord.WARC_RECORD_ID]
 
+        _logger.debug('Writing WARC record {0}.'.format(
+            record.fields['WARC-Type']))
+
         if self._gzip_enabled:
             open_func = wpull.backport.gzip.GzipFile
         else:
             open_func = open
 
+        if os.path.exists(self._filename):
+            before_offset = os.path.getsize(self._filename)
+        else:
+            before_offset = 0
+
         with open_func(self._filename, mode='ab') as out_file:
             for data in record:
                 out_file.write(data)
 
+        after_offset = os.path.getsize(self._filename)
+
+        if self._cdx_filename:
+            raw_file_offset = before_offset
+            raw_file_record_size = after_offset - before_offset
+
+            self._write_cdx_field(
+                record, raw_file_record_size, raw_file_offset
+            )
+
     def close(self):
         '''Close the WARC file and clean up any logging handlers.'''
         if self._log_record:
@@ -338,13 +275,104 @@ class WARCRecorder(BaseRecorder):
             self._log_record.fields['WARC-Target-URI'] = \
                 'urn:X-wpull:log'
 
-            self._log_record.compute_checksum()
+            self.set_length_and_maybe_checksums(self._log_record)
             self.write_record(self._log_record)
 
             self._log_record.block_file.close()
 
+    def _write_cdx_header(self):
+        '''Write the CDX header.
+
+        It writes the fields:
+
+        1. a: original URL
+        2. b: UNIX timestamp
+        3. m: MIME Type from the HTTP Content-type
+        4. s: response code
+        5. k: new style checksum
+        6. S: raw file record size
+        7. V: offset in raw file
+        8. g: filename of raw file
+        9. u: record ID
+        '''
+        with open(self._cdx_filename, mode='a', encoding='utf-8') as out_file:
+            out_file.write(self.CDX_DELIMINATOR)
+            out_file.write(self.CDX_DELIMINATOR.join((
+                'CDX',
+                'a', 'b', 'm', 's',
+                'k', 'S', 'V', 'g',
+                'u'
+            )))
+            out_file.write('\n')
+
+    def _write_cdx_field(self, record, raw_file_record_size, raw_file_offset):
+        '''Write the CDX field if needed.'''
+        if record.fields[WARCRecord.WARC_TYPE] != WARCRecord.RESPONSE \
+        or not re.match(r'application/http; *msgtype *= *response',
+        record.fields[WARCRecord.CONTENT_TYPE]):
+            return
+
+        url = record.fields['WARC-Target-URI']
+
+        _logger.debug('Writing CDX record {0}.'.format(url))
+
+        http_header = record.get_http_header()
+
+        if http_header:
+            mime_type = self.parse_mimetype(
+                http_header.fields.get('Content-Type', '')
+            ) or '-'
+            response_code = str(http_header.status_code)
+        else:
+            mime_type = '-'
+            response_code = '-'
+
+        timestamp = str(int(
+            wpull.util.parse_iso8601_str(record.fields[WARCRecord.WARC_DATE])
+        ))
+
+        checksum = record.fields.get('WARC-Payload-Digest', '')
+
+        if checksum.startswith('sha1:'):
+            checksum = checksum.replace('sha1:', '', 1)
+        else:
+            checksum = '-'
+
+        raw_file_record_size_str = str(raw_file_record_size)
+        raw_file_offset_str = str(raw_file_offset)
+        filename = os.path.basename(self._filename)
+        record_id = record.fields[WARCRecord.WARC_RECORD_ID]
+        fields_strs = (
+            url,
+            timestamp,
+            mime_type,
+            response_code,
+            checksum,
+            raw_file_record_size_str,
+            raw_file_offset_str,
+            filename,
+            record_id
+        )
+
+        with open(self._cdx_filename, mode='a', encoding='utf-8') as out_file:
+            out_file.write(self.CDX_DELIMINATOR.join(fields_strs))
+            out_file.write('\n')
+
+    @classmethod
+    def parse_mimetype(cls, value):
+        '''Return the MIME type from a Content-Type string.
+
+        Returns:
+            str, None: A string in the form ``type/subtype`` or None.
+        '''
+        match = re.match(r'([a-zA-Z0-9-]+/[a-zA-Z0-9-]+)', value)
+
+        if match:
+            return match.group(1)
+
 
 class WARCRecorderSession(BaseRecorderSession):
+    '''WARC Recorder Session.'''
     def __init__(self, recorder, temp_dir=None):
         self._recorder = recorder
         self._request = None
@@ -374,7 +402,9 @@ class WARCRecorderSession(BaseRecorderSession):
         payload_offset = len(request.header())
 
         self._request_record.block_file.seek(0)
-        self._request_record.compute_checksum(payload_offset=payload_offset)
+        self._recorder.set_length_and_maybe_checksums(
+            self._request_record, payload_offset=payload_offset
+        )
         self._recorder.write_record(self._request_record)
 
     def pre_response(self, response):
@@ -393,11 +423,15 @@ class WARCRecorderSession(BaseRecorderSession):
         payload_offset = len(response.header())
 
         self._response_record.block_file.seek(0)
-        self._response_record.compute_checksum(payload_offset=payload_offset)
+        self._recorder.set_length_and_maybe_checksums(
+            self._response_record,
+            payload_offset=payload_offset
+        )
         self._recorder.write_record(self._response_record)
 
 
 class DebugPrintRecorder(BaseRecorder):
+    '''Debugging print recorder.'''
     @contextlib.contextmanager
     def session(self):
         print('Session started')
@@ -408,6 +442,7 @@ class DebugPrintRecorder(BaseRecorder):
 
 
 class DebugPrintRecorderSession(BaseRecorderSession):
+    '''Debugging print recorder session.'''
     def pre_request(self, request):
         print(request)
 
@@ -435,6 +470,7 @@ class PrintServerResponseRecorder(BaseRecorder):
 
 
 class PrintServerResponseRecorderSession(BaseRecorderSession):
+    '''Print Server Response Recorder Session.'''
     def response(self, response):
         print(response.header().decode())
 
@@ -454,6 +490,7 @@ class ProgressRecorder(BaseRecorder):
 
 
 class BaseProgressRecorderSession(BaseRecorderSession):
+    '''Base Progress Recorder Session.'''
     def __init__(self, stream=sys.stderr):
         self._stream = stream
         self._bytes_received = 0
@@ -512,6 +549,11 @@ class BaseProgressRecorderSession(BaseRecorderSession):
 
 
 class DotProgressRecorderSession(BaseProgressRecorderSession):
+    '''Dot Progress Recorder Session.
+
+    This session is responsible for printing dots every few seconds
+    when it receives data.
+    '''
     def __init__(self, dot_interval=2.0, **kwargs):
         super().__init__(**kwargs)
         self._last_flush_time = 0
@@ -536,6 +578,11 @@ class DotProgressRecorderSession(BaseProgressRecorderSession):
 
 
 class BarProgressRecorderSession(BaseProgressRecorderSession):
+    '''Bar Progress Recorder Session.
+
+    This session is responsible for displaying the ASCII bar
+    and stats.
+    '''
     def __init__(self, update_interval=0.5, bar_width=25, **kwargs):
         super().__init__(**kwargs)
         self._last_flush_time = 0
